/*
 * Copyright (C) 2006 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

/** \file file_writer.c simple output plugin for testing purposes */

#include "para.h"
#include "write.h"
#include "string.h"
#include "error.h"

/** data specific to the file writer */
struct private_file_writer_data {
/** the file descriptor of the output file */
int fd;
};

static int file_writer_open(struct writer_node *w)
{
	struct private_file_writer_data *pfwd = para_calloc(
		sizeof(struct private_file_writer_data));
	char *tmp = para_tmpname(), *home = para_homedir(),
		*filename = make_message("%s/.paraslash/%s", home, tmp);

	free(home);
	free(tmp);
	w->private_data = pfwd;
	pfwd->fd = open(filename, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR);
	free(filename);
	if (pfwd->fd >= 0)
		return 8192;
	free(pfwd);
	return -E_FW_OPEN;
}

static int file_writer_write(char *data, size_t nbytes, struct writer_node *wn)
{
	struct private_file_writer_data *pfwd = wn->private_data;
	int ret = write(pfwd->fd, data, nbytes);
	if (ret < 0)
		ret = -E_FW_WRITE;
	return ret;
}

static void file_writer_close(struct writer_node *wn)
{
	struct private_file_writer_data *pfwd = wn->private_data;
	close(pfwd->fd);
	free(pfwd);
}

/** the init function of the file writer */
void file_writer_init(struct writer *w)
{
	w->open = file_writer_open;
	w->write = file_writer_write;
	w->close = file_writer_close;
	w->shutdown = NULL; /* nothing to do */
}
